@@ -30,11 +30,22 @@
 #'#Call it again to change the water depth
 #'\dontrun{
 #'render_water(montereybay,zscale=50,waterdepth=-1000)
+#'render_snapshot()
+#'}
+#'
+#'#Add waterlines
+#'\dontrun{
+#'render_camera(theta=-45)
+#'render_water(montereybay,zscale=50,waterlinecolor="white")
 #'render_snapshot(clear = TRUE)
+#'rgl::rgl.close()
 #'}
 render_water = function(heightmap, waterdepth=0, watercolor="lightblue",
                         zscale=1, wateralpha=0.5, waterlinecolor=NULL, waterlinealpha = 1, 
                         linewidth = 2, remove_water = TRUE) {
+  if(rgl::rgl.cur() == 0) {
+    stop("No rgl window currently open.")
+  }
   if(remove_water) {
     idlist = get_ids_with_labels()
     remove_ids = idlist$id[idlist$raytype %in% c("waterlines", "water")]
@@ -43,8 +54,12 @@ render_water = function(heightmap, waterdepth=0, watercolor="lightblue",
   make_water(heightmap/zscale,waterheight=waterdepth/zscale,wateralpha=wateralpha,watercolor=watercolor)
   if(!is.null(waterlinecolor)) {
     if(all(!is.na(heightmap))) {
-      make_lines(heightmap,basedepth=waterdepth/zscale,linecolor=waterlinecolor,zscale=zscale,linewidth = linewidth,alpha=waterlinealpha,solid=FALSE)
+      rgl::rgl.material(color=waterlinecolor,lit=FALSE)
+      make_lines(fliplr(heightmap),basedepth=waterdepth/zscale,linecolor=waterlinecolor,
+                 zscale=zscale,linewidth = linewidth, alpha=waterlinealpha, solid=FALSE)
     }
-    make_waterlines(heightmap,waterdepth=waterdepth/zscale,linecolor=waterlinecolor,zscale=zscale,alpha=waterlinealpha,linewidth=linewidth)
+    rgl::rgl.material(color=waterlinecolor,lit=FALSE)
+    make_waterlines(heightmap,waterdepth=waterdepth/zscale,linecolor=waterlinecolor,
+                    zscale=zscale,alpha=waterlinealpha,linewidth=linewidth)
   }
 }
\ No newline at end of file
